@@ -6,6 +6,9 @@ from ..constants import *
 
 def main(config: dict):
 	namespace: str = config['namespace']
+	build_datapack: str = config['build_datapack']
+	datapack_functions: str = config['datapack_functions']
+
 	# Stop if not custom block
 	if not any(data.get(VANILLA_BLOCK) for data in config['database'].values()):
 		return
@@ -14,10 +17,10 @@ def main(config: dict):
 	FACING = ["north", "east", "south", "west"]
 	for face in FACING:
 		predicate = {"condition":"minecraft:location_check","predicate":{"block":{"state":{"facing":face}}}}
-		write_to_file(f"{config['build_datapack']}/data/{namespace}/predicate/facing/{face}.json", super_json_dump(predicate))
+		write_to_file(f"{build_datapack}/data/{namespace}/predicate/facing/{face}.json", super_json_dump(predicate))
 
 	# Get rotation function
-	write_to_file(f"{config['datapack_functions']}/custom_blocks/get_rotation.mcfunction", f"""
+	write_to_file(f"{datapack_functions}/custom_blocks/get_rotation.mcfunction", f"""
 # Set up score
 scoreboard players set #rotation {namespace}.data 0
 
@@ -33,30 +36,21 @@ execute if score #rotation {namespace}.data matches 0 if predicate {namespace}:f
 execute if score #rotation {namespace}.data matches 0 if predicate {namespace}:facing/south run scoreboard players set #rotation {namespace}.data 3
 execute if score #rotation {namespace}.data matches 0 if predicate {namespace}:facing/west run scoreboard players set #rotation {namespace}.data 4
 # No more cases for now
-
 """)
-	APPLY_FACING = f"""
-# Apply rotation
-execute if score #rotation {namespace}.data matches 1 run data modify entity @s Rotation[0] set value 180.0f
-execute if score #rotation {namespace}.data matches 2 run data modify entity @s Rotation[0] set value 270.0f
-execute if score #rotation {namespace}.data matches 3 run data modify entity @s Rotation[0] set value 0.0f
-execute if score #rotation {namespace}.data matches 4 run data modify entity @s Rotation[0] set value 90.0f
-"""
 
 	# For each custom block
 	unique_blocks = set()
 	for item, data in config['database'].items():
+		item_name: str = item.replace("_", " ").title()
 
 		# Custom block
 		if data.get(VANILLA_BLOCK):
 			block = data[VANILLA_BLOCK]
 			block_id = block["id"]
-			path = f"{config['datapack_functions']}/custom_blocks/{item}"
+			path = f"{datapack_functions}/custom_blocks/{item}"
+			beautify_name: str = ""
 			if block_id in BLOCKS_WITH_INTERFACES:
-				beautify_name: str = item.replace("_", " ").title()
-				beautify_name = json.dumps({"CustomName":'"' + beautify_name + '"'})
-			else:
-				beautify_name: str = ""
+				beautify_name = json.dumps({"CustomName":'"' + item_name + '"'})
 
 			## Place function	
 			content = ""
@@ -82,11 +76,32 @@ execute if score #rotation {namespace}.data matches 4 run data modify entity @s
 
 			# Add temporary tags and call main function
 			content = f"tag @s add {namespace}.placer\n" + content + f"tag @s remove {namespace}.placer\n"
-			write_to_file(f"{path}/place_main.mcfunction", content)
 
-			## Secondary function
+			# Increment count scores for stats and optimization
 			block_id: str = block_id.split('[')[0].split('{')[0]
 			unique_blocks.add(block_id)
+			content += f"""
+# Increment count scores
+scoreboard players add #total_custom_blocks {namespace}.data 1
+scoreboard players add #total_vanilla_{block_id.replace('minecraft:','')} {namespace}.data 1
+scoreboard players add #total_{item} {namespace}.data 1
+"""
+
+			# Write the file
+			write_to_file(f"{path}/place_main.mcfunction", content)
+
+			# Write the line in stats_custom_blocks
+			write_to_file(
+				f"{datapack_functions}/_stats_custom_blocks.mcfunction",
+				f'tellraw @s [{{"text":"- Total \'{item_name}\': ","color":"gold"}},{{"score":{{"name":"#total_{item}","objective":"{namespace}.data"}},"color":"yellow"}}]\n'
+			)
+			write_to_file(
+				f"{datapack_functions}/_stats_custom_blocks.mcfunction",
+				f'scoreboard players add #total_{item} {namespace}.data 0\n',
+				prepend = True
+			)
+
+			## Secondary function
 			block_id = block_id.replace(":","_")
 			set_custom_model_data = ""
 			if data.get("custom_model_data"):
@@ -107,7 +122,13 @@ data modify entity @s transformation.translation[1] set value 0.003f
 data modify entity @s brightness set value {{block:15,sky:15}}
 """
 			if block["apply_facing"]:
-				content += APPLY_FACING
+				content += f"""
+# Apply rotation
+execute if score #rotation {namespace}.data matches 1 run data modify entity @s Rotation[0] set value 180.0f
+execute if score #rotation {namespace}.data matches 2 run data modify entity @s Rotation[0] set value 270.0f
+execute if score #rotation {namespace}.data matches 3 run data modify entity @s Rotation[0] set value 0.0f
+execute if score #rotation {namespace}.data matches 4 run data modify entity @s Rotation[0] set value 90.0f
+"""
 
 			# Add the commands on placement if any
 			if COMMANDS_ON_PLACEMENT in data:
@@ -117,7 +138,7 @@ data modify entity @s brightness set value {{block:15,sky:15}}
 					content += f"{data[COMMANDS_ON_PLACEMENT]}\n"
 			
 			# If Furnace NBT Recipes is enabled and the block is a furnace, summon the marker
-			if OFFICIAL_LIBS["furnace_nbt_recipes"]["is_used"] and block_id.endswith("_furnace"):
+			if OFFICIAL_LIBS["furnace_nbt_recipes"]["is_used"] and block_id.endswith(("_furnace", "_smoker")):
 				content += '\n# Furnace NBT Recipes\n'
 				content += 'execute align xyz positioned ~.5 ~ ~.5 unless entity @e[type=marker,dx=-1,dy=-1,dz=-1,tag=furnace_nbt_recipes.furnace] run summon marker ~ ~ ~ {Tags:["furnace_nbt_recipes.furnace"]}\n'
 				
@@ -126,7 +147,7 @@ data modify entity @s brightness set value {{block:15,sky:15}}
 		pass
 
 	# Link the custom block library to the datapack
-	smithed_custom_blocks = [(item, data) for item, data in config['database'].items() if data.get("id") == CUSTOM_BLOCK_VANILLA]
+	smithed_custom_blocks = [1 for data in config['database'].values() if data.get("id") == CUSTOM_BLOCK_VANILLA]
 	if smithed_custom_blocks:
 
 		# Change is_used state
@@ -134,10 +155,10 @@ data modify entity @s brightness set value {{block:15,sky:15}}
 			debug("Found custom blocks using CUSTOM_BLOCK_VANILLA in the database, adding 'smithed.custom_block' to the dependencies")
 
 		# Write function tag to link with the library
-		write_to_file(f"{config['build_datapack']}/data/smithed.custom_block/tags/function/event/on_place.json", super_json_dump({"values": [f"{namespace}:custom_blocks/on_place"]}))
+		write_to_file(f"{build_datapack}/data/smithed.custom_block/tags/function/event/on_place.json", super_json_dump({"values": [f"{namespace}:custom_blocks/on_place"]}))
 
 		# Write the slot function
-		write_to_file(f"{config['datapack_functions']}/custom_blocks/on_place.mcfunction", f"execute if data storage smithed.custom_block:main blockApi.__data.Items[0].components.\"minecraft:custom_data\".smithed.block{{from:\"{namespace}\"}} run function {namespace}:custom_blocks/place\n")
+		write_to_file(f"{datapack_functions}/custom_blocks/on_place.mcfunction", f"execute if data storage smithed.custom_block:main blockApi.__data.Items[0].components.\"minecraft:custom_data\".smithed.block{{from:\"{namespace}\"}} run function {namespace}:custom_blocks/place\n")
 
 		# Write the function that will place the custom blocks
 		content = f"tag @s add {namespace}.placer\n"
@@ -145,7 +166,7 @@ data modify entity @s brightness set value {{block:15,sky:15}}
 			if data.get("id") == CUSTOM_BLOCK_VANILLA:
 				content += f"execute if data storage smithed.custom_block:main blockApi{{id:\"{namespace}:{item}\"}} run function {namespace}:custom_blocks/{item}/place_main\n"
 		content += f"tag @s remove {namespace}.placer\n"
-		write_to_file(f"{config['datapack_functions']}/custom_blocks/place.mcfunction", content)
+		write_to_file(f"{datapack_functions}/custom_blocks/place.mcfunction", content)
 
 	# Sort unique blocks
 	unique_blocks = sorted(list(unique_blocks))
@@ -154,13 +175,28 @@ data modify entity @s brightness set value {{block:15,sky:15}}
 	# For each unique block, if the vanilla block is missing, call the destroy function for the group
 	content = "\n"
 	for block_id in unique_blocks:
+		score_check: str = f"score #total_vanilla_{block_id.replace('minecraft:','')} {namespace}.data matches 1.."
 		block_underscore = block_id.replace(":","_")
 		block_id = "#minecraft:cauldrons" if block_id == "minecraft:cauldron" else block_id
-		content += f"execute if entity @s[tag={namespace}.vanilla.{block_underscore}] unless block ~ ~ ~ {block_id} run function {namespace}:custom_blocks/_groups/{block_underscore}\n"
-	write_to_file(f"{config['datapack_functions']}/custom_blocks/destroy.mcfunction", content + "\n")
+		content += f"execute if {score_check} if entity @s[tag={namespace}.vanilla.{block_underscore}] unless block ~ ~ ~ {block_id} run function {namespace}:custom_blocks/_groups/{block_underscore}\n"
+	write_to_file(f"{datapack_functions}/custom_blocks/destroy.mcfunction", content + "\n")
 
 	# For each unique block, make the group function
 	for block_id in unique_blocks:
+
+		# Add a line in the stats_custom_blocks file
+		score_name: str = f"total_vanilla_{block_id.replace('minecraft:','')}"
+		write_to_file(
+			f"{datapack_functions}/_stats_custom_blocks.mcfunction",
+			f'tellraw @s [{{"text":"- Vanilla \'{block_id}\': ","color":"gray"}},{{"score":{{"name":"#{score_name}","objective":"{namespace}.data"}},"color":"white"}}]\n'
+		)
+		write_to_file(
+			f"{datapack_functions}/_stats_custom_blocks.mcfunction",
+			f'scoreboard players add #{score_name} {namespace}.data 0\n',
+			prepend = True
+		)
+
+		# Prepare the group function
 		block_underscore = block_id.replace(":","_")
 		content = "\n"
 
@@ -174,15 +210,16 @@ data modify entity @s brightness set value {{block:15,sky:15}}
 
 				# Add the line if it's the same vanilla block
 				if this_block == block_underscore:
-					content += f"execute if entity @s[tag={namespace}.{item}] run function {namespace}:custom_blocks/{item}/destroy\n"
-		write_to_file(f"{config['datapack_functions']}/custom_blocks/_groups/{block_underscore}.mcfunction", content + "\n")
+					score_check: str = f"score #total_{item} {namespace}.data matches 1.."
+					content += f"execute if {score_check} if entity @s[tag={namespace}.{item}] run function {namespace}:custom_blocks/{item}/destroy\n"
+		write_to_file(f"{datapack_functions}/custom_blocks/_groups/{block_underscore}.mcfunction", content + "\n")
 
 	# For each custom block, make it's destroy function
 	for item, data in config['database'].items():
 		if data.get(VANILLA_BLOCK):
 			block = data[VANILLA_BLOCK]
-			path = f"{config['datapack_functions']}/custom_blocks/{item}"
-			block_id = block["id"].split('[')[0].split('{')[0]
+			path = f"{datapack_functions}/custom_blocks/{item}"
+			block_id: str = block["id"].split('[')[0].split('{')[0]
 			
 			# Destroy function
 			content = f"""
@@ -195,6 +232,16 @@ execute as @e[type=item,nbt={{Item:{{id:"{block_id}"}}}},limit=1,sort=nearest,di
 					content += "\n".join(data[COMMANDS_ON_BREAK]) + "\n"
 				else:
 					content += f"{data[COMMANDS_ON_BREAK]}\n"
+			
+			# Decrease count scores for stats and optimization
+			content += f"""
+# Decrease count scores
+scoreboard players remove #total_custom_blocks {namespace}.data 1
+scoreboard players remove #total_vanilla_{block_id.replace('minecraft:','')} {namespace}.data 1
+scoreboard players remove #total_{item} {namespace}.data 1
+"""
+			
+			# Add the destroy function
 			write_to_file(f"{path}/destroy.mcfunction", content + "\n# Kill the custom block entity\nkill @s\n\n")
 
 			# Replace item function
@@ -230,9 +277,9 @@ execute store result entity @s Item.count byte 1 run scoreboard players get #ite
 	if "minecraft:cauldron" in listed_blocks:
 		listed_blocks.remove("minecraft:cauldron")
 		listed_blocks.append("#minecraft:cauldrons")
-	write_to_file(f"{config['build_datapack']}/data/{namespace}/tags/block/{VANILLA_BLOCKS_TAG}.json", super_json_dump({"values": listed_blocks}))
+	write_to_file(f"{build_datapack}/data/{namespace}/tags/block/{VANILLA_BLOCKS_TAG}.json", super_json_dump({"values": listed_blocks}))
 	predicate = {"condition": "minecraft:location_check", "predicate": {"block": {"blocks": f"#{namespace}:{VANILLA_BLOCKS_TAG}"}}}
-	write_to_file(f"{config['build_datapack']}/data/{namespace}/predicate/check_vanilla_blocks.json", super_json_dump(predicate))
+	write_to_file(f"{build_datapack}/data/{namespace}/predicate/check_vanilla_blocks.json", super_json_dump(predicate))
 	advanced_predicate = {"condition": "minecraft:any_of", "terms": []}
 	for block in unique_blocks:
 		block_underscore = block.replace(":","_")
@@ -240,34 +287,35 @@ execute store result entity @s Item.count byte 1 run scoreboard players get #ite
 			block = "#minecraft:cauldrons"
 		predicate = {"condition": "minecraft:entity_properties", "entity": "this", "predicate": { "nbt": f"{{Tags:[\"{namespace}.vanilla.{block_underscore}\"]}}", "location": { "block": { "blocks": block }}}}
 		advanced_predicate["terms"].append(predicate)
-	write_to_file(f"{config['build_datapack']}/data/{namespace}/predicate/advanced_check_vanilla_blocks.json", super_json_dump(advanced_predicate))
+	write_to_file(f"{build_datapack}/data/{namespace}/predicate/advanced_check_vanilla_blocks.json", super_json_dump(advanced_predicate))
 
 	# Write a destroy check every 2 ticks, every second, and every 5 seconds
 	ore_vanilla_block = VANILLA_BLOCK_FOR_ORES["id"].replace(':', '_')
 	version: str = config['version']
-	write_to_file(f"{config['datapack_functions']}/v{version}/tick_2.mcfunction", f"""
+	score_check: str = f"score #total_custom_blocks {namespace}.data matches 1.."
+	write_to_file(f"{datapack_functions}/v{version}/tick_2.mcfunction", f"""
 # 2 ticks destroy detection
-execute as @e[type=item_display,tag={namespace}.custom_block,tag=!{namespace}.vanilla.{ore_vanilla_block},predicate=!{namespace}:check_vanilla_blocks] at @s run function {namespace}:custom_blocks/destroy
+execute if {score_check} as @e[type=item_display,tag={namespace}.custom_block,tag=!{namespace}.vanilla.{ore_vanilla_block},predicate=!{namespace}:check_vanilla_blocks] at @s run function {namespace}:custom_blocks/destroy
 """)
-	write_to_file(f"{config['datapack_functions']}/v{version}/second.mcfunction", f"""
+	write_to_file(f"{datapack_functions}/v{version}/second.mcfunction", f"""
 # 1 second break detection
-execute as @e[type=item_display,tag={namespace}.custom_block,tag=!{namespace}.vanilla.{ore_vanilla_block},predicate=!{namespace}:advanced_check_vanilla_blocks] at @s run function {namespace}:custom_blocks/destroy
+execute if {score_check} as @e[type=item_display,tag={namespace}.custom_block,tag=!{namespace}.vanilla.{ore_vanilla_block},predicate=!{namespace}:advanced_check_vanilla_blocks] at @s run function {namespace}:custom_blocks/destroy
 """)
-	write_to_file(f"{config['datapack_functions']}/v{version}/second_5.mcfunction", f"""
+	write_to_file(f"{datapack_functions}/v{version}/second_5.mcfunction", f"""
 # 5 seconds break detection
-execute as @e[type=item_display,tag={namespace}.custom_block,predicate=!{namespace}:advanced_check_vanilla_blocks] at @s run function {namespace}:custom_blocks/destroy
+execute if {score_check} as @e[type=item_display,tag={namespace}.custom_block,predicate=!{namespace}:advanced_check_vanilla_blocks] at @s run function {namespace}:custom_blocks/destroy
 """)
 
 
 
 	## Custom ores break detection (if any custom ore)
 	if any(data.get(VANILLA_BLOCK) == VANILLA_BLOCK_FOR_ORES for data in config['database'].values()):
-		write_to_file(f"{config['build_datapack']}/data/common_signals/tags/function/signals/on_new_item.json", super_json_dump({"values": [f"{namespace}:calls/common_signals/new_item"]}))
-		write_to_file(f"{config['datapack_functions']}/calls/common_signals/new_item.mcfunction", f"""
+		write_to_file(f"{build_datapack}/data/common_signals/tags/function/signals/on_new_item.json", super_json_dump({"values": [f"{namespace}:calls/common_signals/new_item"]}))
+		write_to_file(f"{datapack_functions}/calls/common_signals/new_item.mcfunction", f"""
 # If the item is from a custom ore, launch the on_ore_destroyed function
 execute if data entity @s Item.components.\"minecraft:custom_data\".common_signals.temp at @s align xyz run function {namespace}:calls/common_signals/on_ore_destroyed
 """)
-		write_to_file(f"{config['datapack_functions']}/calls/common_signals/on_ore_destroyed.mcfunction", f"""
+		write_to_file(f"{datapack_functions}/calls/common_signals/on_ore_destroyed.mcfunction", f"""
 # Get in a score the item count and if it is a silk touch
 scoreboard players set #item_count {namespace}.data 0
 scoreboard players set #is_silk_touch {namespace}.data 0
@@ -279,6 +327,17 @@ execute as @e[tag={namespace}.custom_block,dx=0,dy=0,dz=0] at @s run function {n
 """)
 	
 
+	# Add line in the stats_custom_blocks file
+	write_to_file(
+		f"{datapack_functions}/_stats_custom_blocks.mcfunction",
+		f'tellraw @s [{{"text":"- Total custom blocks: ","color":"dark_aqua"}},{{"score":{{"name":"#total_custom_blocks","objective":"{namespace}.data"}},"color":"aqua"}}]\n'
+	)
+	write_to_file(
+		f"{datapack_functions}/_stats_custom_blocks.mcfunction",
+		f'scoreboard players add #total_custom_blocks {namespace}.data 0\n',
+		prepend = True
+	)
+
 
 	## Custom blocks using player_head
 	for item, data in config['database'].items():
@@ -288,7 +347,7 @@ execute as @e[tag={namespace}.custom_block,dx=0,dy=0,dz=0] at @s run function {n
 			predicate = {"criteria":{"requirement":{"trigger":"minecraft:placed_block","conditions":{"location": [{"condition": "minecraft:location_check","predicate": {"block": {}}}]}}},"requirements":[["requirement"]],"rewards":{}}
 			predicate["criteria"]["requirement"]["conditions"]["location"][0]["predicate"]["block"]["nbt"] = json.dumps({"components":{"minecraft:custom_data":data.get("custom_data", {})}})
 			predicate["rewards"]["function"] = f"{namespace}:custom_blocks/_player_head/search_{item}"
-			write_to_file(f"{config['build_datapack']}/data/{namespace}/advancement/custom_block_head/{item}.json", super_json_dump(predicate, max_level = -1))
+			write_to_file(f"{build_datapack}/data/{namespace}/advancement/custom_block_head/{item}.json", super_json_dump(predicate, max_level = -1))
 
 			# Make search function
 			content = "# Search where the head has been placed\n"
@@ -298,7 +357,7 @@ execute as @e[tag={namespace}.custom_block,dx=0,dy=0,dz=0] at @s run function {n
 					for z in range(-mid_z, mid_z + 1):
 						content += f"execute positioned ~{x} ~{y} ~{z} if data block ~ ~ ~ components.\"minecraft:custom_data\".{namespace}.{item} run function {namespace}:custom_blocks/{item}/place_main\n"
 			content += f"\n# Advancement\nadvancement revoke @s only {namespace}:custom_block_head/{item}\n\n"
-			write_to_file(f"{config['datapack_functions']}/custom_blocks/_player_head/search_{item}.mcfunction", content)
+			write_to_file(f"{datapack_functions}/custom_blocks/_player_head/search_{item}.mcfunction", content)
 
 	info("All customs blocks are now placeable and destroyable!")
 
