@@ -44,6 +44,12 @@ HOVER_FURNACE_FONT = get_font(0x000E)
 INVISIBLE_ITEM_FONT = get_font(0x000F)	# Invisible item to place
 INVISIBLE_ITEM_WIDTH = INVISIBLE_ITEM_FONT + MICRO_NONE_FONT
 
+HOVER_EQUIVALENTS: dict[str, str] = {
+	SHAPED_2X2_FONT: HOVER_SHAPED_2X2_FONT,
+	SHAPED_3X3_FONT: HOVER_SHAPED_3X3_FONT,
+	FURNACE_FONT: HOVER_FURNACE_FONT,
+}
+
 # Global variables
 next_craft_font = 0x8000
 font_providers = []
